package mif18.prj;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import junit.framework.TestCase;

import org.openjena.riot.RiotReader;
import org.openjena.riot.RiotWriter;

import com.hp.hpl.jena.datatypes.xsd.XSDDatatype;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;

/**
 * Classe contenant des exemples de code pour les entrées/sorties RDF.
 * 
 * @author Emmanuel Coquery
 */
public class ARQTest extends TestCase {

	private static final String BASE_URI = "http://www.univ-lyon1.fr/M1/MIF18/films/";

	/**
	 * Le repertoire dans lequel on ecrit. Eventuellement cree lors du setup des
	 * tests.
	 */
	private File outputDir;

	/**
	 * Vérifie que le fichier
	 * 
	 * @param dir
	 * @return
	 * @throws IOException
	 */
	private static File createCheckDir(File dir) throws IOException {
		if (!dir.exists()) {
			dir.mkdir();
		} else if (!dir.isDirectory()) {
			throw new IOException(dir.getAbsolutePath()
					+ " n'est pas un repertoire");
		}
		return dir;
	}

	private static Node uri(String end) {
		return Node.createURI(BASE_URI + end);
	}

	private static Node string(String valeur) {
		return Node.createLiteral(valeur);
	}

	private static Node integer(int valeur) {
		return Node.createLiteral(String.valueOf(valeur), XSDDatatype.XSDint);
	}

	private static Node date(int jour, int mois, int annee)
			throws DatatypeConfigurationException {
		XMLGregorianCalendar xd = DatatypeFactory.newInstance()
				.newXMLGregorianCalendar();
		xd.clear();
		xd.setYear(annee);
		xd.setMonth(mois);
		xd.setDay(jour);
		return Node.createLiteral(xd.toXMLFormat(), XSDDatatype.XSDdate);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		if (outputDir == null) {
			File target = createCheckDir(new File("target"));
			outputDir = createCheckDir(new File(target, "testoutput"));
		}
	}

	/**
	 * Crée un itérateur sur des triplets puis stocke les dits triplets dans un fichier au format N-Triple.
	 * @throws DatatypeConfigurationException
	 * @throws IOException
	 */
	public void testWriteTriples() throws DatatypeConfigurationException, IOException {
		Collection<Triple> triplets = new ArrayList<Triple>();
		Node eastwood = uri("personne/nm0000142");
		triplets.add(new Triple(eastwood, uri("nom"), string("Clint Eastwood")));
		triplets.add(new Triple(eastwood, uri("naissance"), date(31, 05, 1930)));
		triplets.add(new Triple(eastwood, uri("taille"), integer(188)));
		Node million_ = uri("film/tt0405159");
		triplets.add(new Triple(million_, uri("titre"),
				string("Million Dollar Baby")));
		triplets.add(new Triple(million_, uri("realisateur"), eastwood));
		Node fdunn = uri("role/ch0004237");
		triplets.add(new Triple(fdunn, uri("acteur"), eastwood));
		triplets.add(new Triple(fdunn, uri("film"), million_));
		triplets.add(new Triple(fdunn, uri("nom"), string("Frank Dunn")));
		Node warner = uri("societe/co0026840");
		triplets.add(new Triple(warner, uri("nom"),
				string("Warner Bros. Pictures")));
		triplets.add(new Triple(warner, uri("pays"), string("US")));
		triplets.add(new Triple(million_, uri("produit_par"), warner));
		Iterator<Triple> iter = triplets.iterator();
		File million_nt = new File(outputDir,"million.nt");
		OutputStream out = new FileOutputStream(million_nt);
		RiotWriter.writeTriples(out, iter);
		out.close();
	}
	
	/**
	 * Lit et compte les triplets stockés dans un fichier au format N-Triple
	 */
	public void testReadTriples() {
		InputStream is = getClass().getResourceAsStream("/million_b.nt");
		CountingTripleSink compteur = new CountingTripleSink();
		RiotReader.createParserNTriples(is, compteur).parse();
		assertEquals(10, compteur.getCombien());
	}
}
